@@ -10,10 +10,18 @@
  *   (at your option) any later version.                                   *
  ***************************************************************************/
 
+#include <QDataStream>
 #include <QFile>
+#include <QHash>
 #include "board.h"
+#include "guess.h"
 
-QMap<quint64, QList<Square> > ecoGuessPositions;
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
+
+QHash<quint64, QList<Square> > ecoGuessPositions;
 
 Board::Board()
     : m_hashValue(0)
@@ -56,6 +64,17 @@ bool Board::fromFen(const QString& fen)
     return false;
 }
 
+bool Board::from64Char(const QString& qcharboard)
+{
+    if (BitBoard::from64Char(qcharboard))
+    {
+        createHash();
+        m_squareAnnotation = "";
+        return true;
+    }
+    return false;
+}
+
 void Board::setAt(Square s, Piece p)
 {
     hashPiece(s, pieceAt(s));
@@ -78,6 +97,11 @@ void Board::setToMove(Color c)
     BitBoard::setToMove(c);
 }
 
+void Board::swapToMove()
+{
+    setToMove(oppositeColor(toMove()));
+}
+
 bool Board::doIt(const Move& m, bool undo)
 {
     if(m.isNullMove())
@@ -259,6 +283,32 @@ quint64 Board::getHashValue() const
     return m_hashValue;
 }
 
+void Board::setSquareAnnotation(QString squareAnnotation)
+{
+    m_squareAnnotation = squareAnnotation;
+}
+
+QString Board::squareAnnotation() const
+{
+    return m_squareAnnotation;
+}
+
+void Board::setArrowAnnotation(QString arrowAnnotation)
+{
+    m_arrowAnnotation = arrowAnnotation;
+}
+
+QString Board::arrowAnnotation() const
+{
+    return m_arrowAnnotation;
+}
+
+int Board::ScoreMaterial() const
+{
+    QString fen = toFen();
+    return Guess::scorePosFromFen(fen.toLatin1());
+}
+
 Board getStandardStartBoard()
 {
     Board b;
