@@ -13,6 +13,11 @@
 #include "filter.h"
 #include <QtDebug>
 
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
+
 Filter::Filter(Database* database) : QThread()
 {
     m_database = database;
