@@ -11,10 +11,16 @@
 #include <QtDebug>
 #include <QFile>
 #include "game.h"
+#include "tags.h"
 
-const QRegExp Game::emt("\\[%emt\\s*(\\d:\\d\\d:\\d\\d)\\]");
-const QRegExp Game::clk("\\[%clk\\s*(\\d:\\d\\d:\\d\\d)\\]");
-const QRegExp Game::egt("\\[%egt\\s*(\\d:\\d\\d:\\d\\d)\\]");
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
+
+const QRegExp Game::emt("\\[%emt\\s*(\\d:\\d\\d?:\\d\\d)\\]");
+const QRegExp Game::clk("\\[%clk\\s*(\\d:\\d\\d?:\\d\\d)\\]");
+const QRegExp Game::egt("\\[%egt\\s*(\\d:\\d\\d?:\\d\\d)\\]");
 const QRegExp Game::csl("\\[%csl\\s*([^\\]]*)\\]");
 const QRegExp Game::cal("\\[%cal\\s*([^\\]]*)\\]");
 
@@ -115,6 +121,55 @@ MoveId Game::addMove(const QString& sanMove, const QString& annotation, NagSet n
     return NO_MOVE;
 }
 
+MoveId Game::addMoveFrom64Char(const QString &qcharboard)
+{
+    QStringList l = qcharboard.split(' ');
+    if (l.size() < 30) return NO_MOVE;
+    Char64Relation relation = (Char64Relation) l[C64_GAME_RELATION].toInt();
+    QString s=l[C64_PP_LAST_MOVE];
+    s.remove('+');
+    s.remove('#');
+
+    QString t = l[C64_ELAPSED_TIME_LAST_MOVE];
+    t.remove("(");
+    t.remove(")");
+    QStringList tl = t.split(':');
+    QString emt;
+    if (tl.size()>=2)
+    {
+        emt = QString("[%emt 0:%1:%2]").arg(tl[0],-2,'0').arg(tl[1],-2,'0');
+    }
+
+    if (relation == C64_REL_PLAY_OPPONENT_MOVE)
+    {
+        if (s=="none")
+        {
+            return NO_MOVE;
+        }
+        else
+        {
+            if (!emt.isEmpty())
+            {
+                setAnnotation(emt);
+            }
+            return CURRENT_MOVE;
+        }
+    }
+    else
+    {
+        Color thisMoveColor = (l[C64_COLOR_TO_MOVE]=="W") ? Black : White;
+        if (thisMoveColor == board().toMove())
+        {
+            MoveId moveId = addMove(s,emt);
+            return moveId;
+        }
+        else
+        {
+            return NO_MOVE;
+        }
+    }
+}
+
 bool Game::mergeNode(Game& otherGame)
 {
     SaveRestoreMove saveThis(*this);
@@ -143,7 +198,7 @@ bool Game::mergeVariations(Game& otherGame)
             {
                 ok = false;
             }
-            otherGame.moveToId(otherCurrent);
+            otherGame.dbMoveToId(otherCurrent);
             if(!ok)
             {
                 break;
@@ -277,7 +332,7 @@ void Game::dbMergeWithGame(const Game& g)
     {
         backward();
         // merge othergame starting with otherMergeNode into variation starting from m_currentNode
-        otherGame.moveToId(otherMergeNode);
+        otherGame.dbMoveToId(otherMergeNode);
         otherGame.forward();
         if(atLineEnd())
         {
@@ -287,14 +342,14 @@ void Game::dbMergeWithGame(const Game& g)
         {
             mergeAsVariation(otherGame);
         }
-        otherGame.moveToId(otherMergeNode);
+        otherGame.dbMoveToId(otherMergeNode);
         mergeVariations(otherGame);
     }
 
     // undo changes
     if(saveNode != m_currentNode)
     {
-        moveToId(saveNode);
+        dbMoveToId(saveNode);
     }
     compact();
 }
@@ -406,7 +461,7 @@ bool Game::findNextMove(Square from, Square to, PieceType promotionPiece)
                 if(m.from() == from && m.to() == to &&
                         ((promotionPiece == None) || ((m.isPromotion() && (pieceType(m.promotedPiece()) == promotionPiece)))))
                 {
-                    moveToId(*i);
+                    dbMoveToId(*i);
                     return true;
                 }
             }
@@ -530,7 +585,7 @@ MoveId Game::dbAddVariation(const MoveList& moveList, const QString& annotation)
     {
         varStart = dbAddMove(moveList.first());
     }
-    moveToId(varStart);
+    dbMoveToId(varStart);
     for(int i = start; i < moveList.count(); ++i)
     {
         dbAddMove(moveList[i]);
@@ -540,7 +595,7 @@ MoveId Game::dbAddVariation(const MoveList& moveList, const QString& annotation)
     {
         dbSetAnnotation(annotation);
     }
-    moveToId(currentPosition);
+    dbMoveToId(currentPosition);
     return varStart;
 }
 
@@ -563,7 +618,7 @@ MoveId Game::dbAddVariation(const QString& sanMove, const QString& annotation, N
 
 void Game::dbPromoteVariation(MoveId variation)
 {
-    MoveId currentNode = m_currentNode;	// Save current move
+    SaveRestoreMove saveNode(*this);
 
     // Find first move of the variation
     while(!atLineStart(variation))
@@ -580,8 +635,6 @@ void Game::dbPromoteVariation(MoveId variation)
     // Swap main line and the variation
     int index = m_moveNodes[parent].variations.indexOf(variation);
     qSwap(m_moveNodes[parent].nextNode, m_moveNodes[parent].variations[index]);
-
-    moveToId(currentNode);	// Restore current move
 }
 
 bool Game::promoteVariation(MoveId variation)
@@ -606,7 +659,7 @@ bool Game::removeVariation(MoveId variation)
     Game state = *this;
     MoveId parentNode = m_moveNodes[variation].parentNode;
     removeNode(variation);
-    moveToId(parentNode);
+    dbMoveToId(parentNode);
 
     QList<MoveId> &vars = m_moveNodes[m_currentNode].variations;
     vars.removeAt(vars.indexOf(variation));
@@ -642,9 +695,8 @@ void Game::truncateVariationAfterNextIllegalPosition()
     }
 }
 
-void Game::truncateVariation(Position position)
+void Game::dbTruncateVariation(Position position)
 {
-    Game state = *this;
     if(position == AfterMove)
     {
         MoveId node = m_moveNodes[m_currentNode].nextNode;
@@ -664,7 +716,7 @@ void Game::truncateVariation(Position position)
             backward();
             forward();
         }
-        MoveId current = m_currentNode;
+        SaveRestoreMove saveNode(*this);
         MoveNode firstNode;
         firstNode.nextNode = m_currentNode;
         firstNode.SetPly(m_moveNodes[m_currentNode].Ply() - 1);
@@ -687,9 +739,14 @@ void Game::truncateVariation(Position position)
             m_tags[TagNameFEN] = m_startingBoard.toFen();
             m_tags[TagNameSetUp] = "1";
         }
-        moveToId(current);
     }
     compact();
+}
+
+void Game::truncateVariation(Position position)
+{
+    Game state = *this;
+    dbTruncateVariation(position);
     emit signalGameModified(true, state, tr("Truncate variation"));
 }
 
@@ -1432,6 +1489,16 @@ MoveId Game::currentMove() const
     return m_currentNode;
 }
 
+MoveId Game::lastMove() const
+{
+    MoveId moveId = 0;
+    while((m_moveNodes[moveId].nextNode != NO_MOVE))
+    {
+        moveId = m_moveNodes[moveId].nextNode;
+    }
+    return moveId;
+}
+
 const QList<MoveId>& Game::variations() const
 {
     return m_moveNodes[m_currentNode].variations;
@@ -1492,11 +1559,11 @@ int Game::moveByPly(int diff)
     }
 }
 
-void Game::moveToId(MoveId moveId)
+bool Game::dbMoveToId(MoveId moveId)
 {
     if(nodeValid(moveId) == NO_MOVE)
     {
-        return;
+        return false;
     }
 
     if (m_currentNode != moveId)
@@ -1519,7 +1586,15 @@ void Game::moveToId(MoveId moveId)
         }
     }
 
-    indicateAnnotationsOnBoard(moveId);
+    return true;
+}
+
+void Game::moveToId(MoveId moveId)
+{
+    if (dbMoveToId(moveId))
+    {
+        indicateAnnotationsOnBoard(moveId);
+    }
 }
 
 Move Game::move(MoveId moveId) const
@@ -1537,12 +1612,12 @@ void Game::moveToEnd()
     // Move out of variations to mainline
     while(m_moveNodes[m_currentNode].parentNode != NO_MOVE)
     {
-        moveToId(m_moveNodes[m_currentNode].parentNode);
+        dbMoveToId(m_moveNodes[m_currentNode].parentNode);
     }
     // Now move forward to the end of the game
-    while(m_moveNodes[m_currentNode].nextNode != NO_MOVE)
+    if (!forward(999))
     {
-        forward(1);
+        indicateAnnotationsOnBoard(m_currentNode);
     }
 }
 
@@ -1595,7 +1670,7 @@ void Game::enterVariation(const MoveId& moveId)
 void Game::removeNode(MoveId moveId)
 {
     MoveId node = nodeValid(moveId);
-    if(node != NO_MOVE)
+    if(node > ROOT_NODE)
     {
         m_variationStartAnnotations.remove(node);
         m_annotations.remove(node);
@@ -1643,7 +1718,12 @@ QString Game::tag(const QString& tag) const
     return m_tags[tag];
 }
 
-const QMap<QString, QString>& Game::tags() const
+bool Game::hasTag(const QString& tag) const
+{
+    return m_tags.contains(tag);
+}
+
+const TagMap& Game::tags() const
 {
     return m_tags;
 }
@@ -1716,7 +1796,7 @@ QString Game::moveToSan(MoveStringFlags flags, NextPreviousMove nextPrevious, Mo
     if(boardNode != m_currentNode)
     {
         saveNode = m_currentNode;
-        moveToId(boardNode);
+        dbMoveToId(boardNode);
     }
 
     // Move number
@@ -1749,7 +1829,7 @@ QString Game::moveToSan(MoveStringFlags flags, NextPreviousMove nextPrevious, Mo
     // Restore previous position
     if(saveNode != NO_MOVE)
     {
-        moveToId(saveNode);
+        dbMoveToId(saveNode);
     }
 
     return san;
@@ -1940,12 +2020,39 @@ QString Game::ecoClassify() const
     return QString();
 }
 
+void Game::scoreMaterial(QList<double>& scores) const
+{
+    Game g = *this;
+    g.moveToStart();
+    scores.clear();
+    int score = g.board().ScoreMaterial();
+    scores.append(score);
+    while(g.forward())
+    {
+        int score = g.board().ScoreMaterial();
+        scores.append(score);
+    }
+}
+
 bool Game::isEcoPosition() const
 {
     quint64 key = m_currentBoard.getHashValue();
     return (m_ecoPositions.contains(key));
 }
 
+QString Game::findEcoNameDetailed(QString eco)
+{
+    foreach(QString actualEco, m_ecoPositions.values())
+    {
+        if (actualEco.startsWith(eco))
+        {
+            QString opName = actualEco.section(" ",1);
+            return opName;
+        }
+    }
+    return QString();
+}
+
 QString Game::findEcoName(QString eco)
 {
     foreach(QString actualEco, m_ecoPositions.values())
@@ -1953,6 +2060,10 @@ QString Game::findEcoName(QString eco)
         if (actualEco.startsWith(eco))
         {
             QString opName = actualEco.section(" ",1);
+            if (opName.contains(':'))
+            {
+                opName = opName.section(":",0,0);
+            }
             return opName;
         }
     }
