@@ -15,7 +15,9 @@
 #include "board.h"
 #include "movelist.h"
 #include "nag.h"
+#include "result.h"
 
+#define ROOT_NODE 0
 #define NO_MOVE -1
 #define CURRENT_MOVE -2
 #define CURRENT_VARIATION -3
@@ -49,6 +51,11 @@ class SaveRestoreMoveCompact;
 
 */
 
+typedef QHash<QString, QString> TagMap;
+typedef QHashIterator<QString, QString> TagMapIterator;
+
+typedef quint32 GameId;
+
 class Game : public QObject
 {
     Q_OBJECT
@@ -103,8 +110,6 @@ public :
     QString toFen() const;
     /** @return current position in human readable FEN */
     QString toHumanFen() const;
-    /** @return whether the current position is in the mainline */
-    bool isMainline(MoveId moveId = CURRENT_MOVE) const;
     /** @return start position of game */
     Board startingBoard() const;
     /** @return game result */
@@ -169,6 +174,8 @@ public :
     // **** tree information methods *****
     /** @return number of siblings of current node */
     int numberOfSiblings(MoveId moveId = CURRENT_MOVE) const;
+    /** @return whether the current position is in the mainline */
+    bool isMainline(MoveId moveId = CURRENT_MOVE) const;
     /** @return whether the game is currently at the start position */
     bool atLineStart(MoveId moveId = CURRENT_MOVE) const;
     bool atGameStart(MoveId moveId = CURRENT_MOVE) const;
@@ -211,6 +218,7 @@ public :
     /** Moves by given ply, returns actual ply reached */
     int moveByPly(int diff);
     /** Moves to the position corresponding to the given move id */
+    bool dbMoveToId(MoveId moveId);
     void moveToId(MoveId moveId);
     /** Move forward the given number of moves, returns actual number of moves made */
     int forward(int count = 1);
@@ -236,6 +244,8 @@ public :
     MoveId dbAddMove(const QString& sanMove, const QString& annotation = QString(), NagSet nags = NagSet());
     /** Adds a move at the current position, returns the move id of the added move */
     MoveId addMove(const QString& sanMove, const QString& annotation = QString(), NagSet nags = NagSet());
+    /** Adds a move at the current position, returns the move id of the added move */
+    MoveId addMoveFrom64Char(const QString& qcharboard);
     /** Replace the move after the current position */
     bool replaceMove(const Move& move, const QString& annotation = QString(), NagSet nags = NagSet(), bool bReplace = true);
     /** Replace the move after the current position */
@@ -279,6 +289,7 @@ public :
     bool removeVariation(MoveId variation);
     /** Removes all variations and mainline moves after the current position,
     * or before the current position if @p position == BeforeMove */
+    void dbTruncateVariation(Position position = AfterMove);
     void truncateVariation(Position position = AfterMove);
     /** Removes all tags and moves */
     void clear();
@@ -299,9 +310,11 @@ public :
     /** @return value of the given tag */
     QString tag(const QString& tag) const;
     /** @return a map of all tags in the game */
-    const QMap<QString, QString> &tags() const;
+    const TagMap &tags() const;
     /** Sets the value of the given tag */
     void setTag(const QString& tag, const QString& value);
+    /** Query existance of tag */
+    bool hasTag(const QString& tag) const;
     /** Removes a tag */
     void removeTag(const QString& tag);
     /** Set the game result */
@@ -315,6 +328,9 @@ public :
     /** @return true if the move @p from @p to is already in a variation */
     bool currentNodeHasVariation(Square from, Square to) const;
 
+    /** Evaluate a list of scores for the complete game (mainline only) */
+    void scoreMaterial(QList<double> &scores) const;
+
     /** @return ECO code for the game */
     QString ecoClassify() const;
     /** @return true if current pos is in the ECO list */
@@ -322,6 +338,7 @@ public :
 
     /** Method that loads a file containing ECO classifications for use by the ecoClassify method. Returns true if successful */
     static bool loadEcoFile(const QString& ecoFile);
+    static QString findEcoNameDetailed(QString eco);
     static QString findEcoName(QString eco);
 
     /* Debug */
@@ -347,6 +364,10 @@ public :
     int resultAsInt() const;
     void setStartingBoard(const Board &startingBoard, QString text);
 
+    /** Removes the node at @p moveId */
+    void removeNode(MoveId moveId = CURRENT_MOVE);
+
+    MoveId lastMove() const;
 protected:
     /** Find the point in the this game where @p otherGame fits in the next time.
         @retval Node from where the merging shall start in other game */
@@ -407,13 +428,11 @@ private:
     AnnotationMap m_annotations;
 
     /** Map keeping pgn tags of the game */
-    QMap<QString, QString> m_tags;
+    TagMap m_tags;
 
     // **** memory  management methods ****
     /** Remove all removed nodes */
     void compact();
-    /** Removes the node at @p moveId */
-    void removeNode(MoveId moveId = CURRENT_MOVE);
 
     /** Checks if a moveId is valid, returns the moveId if it is, 0 if not */
     MoveId nodeValid(MoveId moveId = CURRENT_MOVE) const;
@@ -436,7 +455,7 @@ public:
     }
     ~SaveRestoreMove()
     {
-        m_saveGame->moveToId(m_saveMoveValue);
+        m_saveGame->dbMoveToId(m_saveMoveValue);
     }
 private:
     Game* m_saveGame;
@@ -453,7 +472,7 @@ public:
     }
     ~SaveRestoreMoveCompact()
     {
-        m_saveGame->moveToId(m_saveMoveValue);
+        m_saveGame->dbMoveToId(m_saveMoveValue);
         m_saveGame->compact();
     }
 private:
