@@ -10,6 +10,11 @@
 
 #include "nag.h"
 
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
+
 QStringList NagSet::g_nagStringListLong;
 
 void NagSet::addNag(Nag nag)
@@ -295,7 +300,7 @@ void NagSet::InitNagStringListLong()
                         tr("Dubious Move") <<
                         tr("Only Move") <<
                         tr("Only Move") <<
-                        tr("Blunder") <<
+                        tr("Worst Move") <<
                         tr("Equal") <<
                         tr("Equal") <<
                         tr("Equal") <<
