@@ -15,6 +15,12 @@
 #include "boardview.h"
 #include "output.h"
 #include "settings.h"
+#include "tags.h"
+
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
 
 QMap<Output::OutputType, QString> Output::m_outputMap;
 
@@ -521,7 +527,7 @@ QString Output::writeMainLine(MoveId upToNode)
                 for(int i = 0; i < variations.size(); ++i)
                 {
                     // *** Enter variation i, and write the rest of the moves
-                    m_game.moveToId(variations[i]);
+                    m_game.dbMoveToId(variations[i]);
                     text += writeVariation();
                 }
                 if(hasNext && m_options.getOptionAsBool("ColumnStyle"))
@@ -530,7 +536,7 @@ QString Output::writeMainLine(MoveId upToNode)
                 }
             }
             m_dirtyBlack = true;
-            m_game.moveToId(m_game.parentMove());
+            m_game.dbMoveToId(m_game.parentMove());
         }
         m_game.forward();
     }
@@ -571,13 +577,13 @@ QString Output::writeVariation()
                 for(int i = 0; i < variations.size(); ++i)
                 {
                     // *** Enter variation i, and write the rest of the moves
-                    m_game.moveToId(variations[i]);
+                    m_game.dbMoveToId(variations[i]);
                     text += writeVariation();
                 }
                 mustAddStart = true;
             }
             m_dirtyBlack = true;
-            m_game.moveToId(m_game.parentMove());
+            m_game.dbMoveToId(m_game.parentMove());
         }
         m_game.forward();
     }
@@ -691,7 +697,7 @@ QString Output::writeGameComment(QString comment) const
 QString Output::writeAllTags() const
 {
     QString text;
-    QMap<QString, QString> tags = m_game.tags();
+    TagMap tags = m_game.tags();
     // write standard tags
     for(int i = 0; i < 7; ++i)
     {
@@ -699,15 +705,17 @@ QString Output::writeAllTags() const
         tags.remove(StandardTags[i]);
     }
 
+    QList<QString> keys = tags.keys();
+    qSort(keys);
+
     // write other tags written in ascii order, as suggested by standard
-    QMapIterator<QString, QString> it(tags);
-    while(it.hasNext())
+    foreach(QString key,keys)
     {
-        it.next();
+        QString value = tags.value(key);
         // workaround for problems with IndexItem implementation
-        if(!it.value().isEmpty() && it.value() != "?" && it.value() != PDInvalidDate.asString() && it.key() != "Length")
+        if(!value.isEmpty() && value != "?" && value != PDInvalidDate.asString() && key != "Length")
         {
-            text += writeTag(it.key(), it.value());
+            text += writeTag(key, value);
         }
     }
     return text;
@@ -716,7 +724,7 @@ QString Output::writeAllTags() const
 QString Output::writeBasicTagsHTML() const
 {
     QString text;
-    QMap<QString, QString> tags = m_game.tags();
+    TagMap tags = m_game.tags();
 
     QString eco = tags[TagNameECO].left(3);
     if(eco == "?")
@@ -814,7 +822,7 @@ QString Output::outputGame(const Game* g, bool upToCurrentMove)
     text += m_endTagMap[MarkupNotationBlock];
     text += m_startTagMap[MarkupResult] + m_game.tag("Result") + m_endTagMap[MarkupResult];
 
-    m_game.moveToId(id);
+    m_game.dbMoveToId(id);
 
     return text;
 }
