@@ -13,13 +13,18 @@
 #include <QDir>
 #include <QStringList>
 #include <QtDebug>
-#include <QDesktopServices>
 
 #include "board.h"
 #include "nag.h"
 
 #include "pgndatabase.h"
 #include "settings.h"
+#include "tags.h"
+
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
 
 PgnDatabase::PgnDatabase(bool b64bit) :
     Database(),
@@ -53,9 +58,9 @@ void PgnDatabase::parseGame()
     skipMoves();
 }
 
-bool PgnDatabase::readIndexFile(QDataStream &in, volatile bool* breakFlag)
+bool PgnDatabase::readIndexFile(QDataStream &in, volatile bool* breakFlag, short version)
 {
-    return (index()->read(in, breakFlag));
+    return (index()->read(in, breakFlag, version));
 }
 
 bool PgnDatabase::writeIndexFile(QDataStream& out) const
@@ -68,17 +73,12 @@ QString PgnDatabase::offsetFilename(const QString& filename) const
     QFileInfo fi = QFileInfo(filename);
     QString basefile = fi.completeBaseName();
     basefile.append(".cxi");
-#if QT_VERSION < 0x050000
-    QString dataPath = QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation) + "/chessdata";
-#else
-    QString dataPath = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation) + "/chessdata";
-#endif
-    QString dir = AppSettings->value("/General/DefaultDataPath", dataPath).toString();
-    QString indexPath = dir + "/index";
-    return(indexPath + "/" + basefile);
+    QString dir = AppSettings->commonDataPath();
+    QString indexPath = dir + QDir::separator() + "index";
+    return(indexPath + QDir::separator() + basefile);
 }
 
-bool PgnDatabase::readOffsetFile(const QString& filename, volatile bool *breakFlag)
+bool PgnDatabase::readOffsetFile(const QString& filename, volatile bool *breakFlag, bool& bUpdate)
 {
     if(!AppSettings->getValue("/General/useIndexFile").toBool())
     {
@@ -158,7 +158,8 @@ bool PgnDatabase::readOffsetFile(const QString& filename, volatile bool *breakFl
 
     in >> magic;
 
-    readIndexFile(in, breakFlag);
+    readIndexFile(in, breakFlag, version);
+    bUpdate = (version < 2);
 
     unsigned short finalMagic;
     in >> finalMagic;
@@ -191,7 +192,7 @@ bool PgnDatabase::writeOffsetFile(const QString& filename) const
 
     QDataStream out(&file);
 
-    short version = 1;
+    short version = 2;
     unsigned short magic = 0xce55;
     int streamVersion = out.version();
 
@@ -234,9 +235,15 @@ bool PgnDatabase::writeOffsetFile(const QString& filename) const
 
 bool PgnDatabase::parseFile()
 {
-    if(readOffsetFile(m_filename, &m_break))
+    bool bUpdate = false;
+    if(readOffsetFile(m_filename, &m_break, bUpdate))
     {
         m_count = m_allocated;
+        emit progress(99);
+        if (bUpdate)
+        {
+            writeOffsetFile(m_filename);
+        }
         emit progress(100);
         return true;
     }
@@ -674,7 +681,7 @@ void PgnDatabase::parseToken(Game* game, const QString& token)
         m_newVariation = true;
         break;
     case ')':
-        game->moveToId(game->parentMove());
+        game->dbMoveToId(game->parentMove());
         game->forward();
         m_newVariation = false;
         m_variation = 0;
