@@ -6,6 +6,11 @@
 #include <QFile>
 #include <QList>
 
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
+
 // Parsed data containers
 static QMap<quint64, QString> ecoPositions;
 static QMap<quint64, QString> ecoNames;
@@ -172,8 +177,8 @@ bool compileAsciiEcoFile(const QString& filenameIn, QString filenameOut, QString
         return false;
     }
 
-    filenameOut = QFileInfo(filenameIn).absolutePath()+"/"+filenameOut;
-    gtmFile = QFileInfo(filenameIn).absolutePath()+"/"+gtmFile;
+    filenameOut = QFileInfo(filenameIn).absolutePath() + QDir::separator() + filenameOut;
+    gtmFile = QFileInfo(filenameIn).absolutePath() + QDir::separator() + gtmFile;
 
     // Write out the main ECO file
     QFile file(filenameOut);
