@@ -22,6 +22,11 @@
 #include <QApplication>
 #include <QSizePolicy>
 
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
+
 using namespace Qt;
 
 const int CoordinateSize = 16;
@@ -33,6 +38,7 @@ BoardView::BoardView(QWidget* parent, int flags) : QWidget(parent),
     m_hoverSquare(InvalidSquare),
     m_hiFrom(InvalidSquare), m_hiTo(InvalidSquare),
     m_currentFrom(InvalidSquare), m_currentTo(InvalidSquare),
+    m_storedFrom(InvalidSquare), m_storedTo(InvalidSquare),
     m_atLineEnd(true),
     m_flags(flags),
     m_coordinates(false), m_dragged(Empty), m_clickUsed(false), m_wheelCurrentDelta(0),
@@ -62,12 +68,19 @@ void BoardView::setFlags(int flags)
     m_flags = flags;
 }
 
+int BoardView::flags() const
+{
+    return m_flags;
+}
+
 void BoardView::setBoard(const Board& value, int from, int to, bool atLineEnd)
 {
     m_clickUsed = true;
     m_board = value;
     m_currentFrom = from;
     m_currentTo = to;
+    m_storedFrom = InvalidSquare;
+    m_storedTo = InvalidSquare;
     m_atLineEnd = atLineEnd;
     m_hiFrom = m_hiTo = InvalidSquare;
     if(underMouse())
@@ -77,6 +90,13 @@ void BoardView::setBoard(const Board& value, int from, int to, bool atLineEnd)
     update();
 }
 
+void BoardView::setStoredMove(int from, int to)
+{
+    m_storedFrom = from;
+    m_storedTo = to;
+    update();
+}
+
 Board BoardView::board() const
 {
     return m_board;
@@ -92,6 +112,11 @@ void BoardView::showMoveIndicator(bool visible)
     m_showMoveIndicator = visible;
 }
 
+void BoardView::showCoordinates(bool visible)
+{
+    m_coordinates = visible;
+}
+
 void BoardView::drawSquares(QPaintEvent* event)
 {
     QPainter p(this);
@@ -215,6 +240,11 @@ void BoardView::drawHiliting(QPaintEvent* event)
 
         QPoint pos = posFromSquare(square);
 
+        if(square == m_storedFrom || square == m_storedTo)
+        {
+            drawHiliteSquare(pos, BoardTheme::StoredMove);
+        }
+
         if(m_showCurrentMove==1)
         {
             if(square == m_currentFrom || square == m_currentTo)
@@ -254,6 +284,15 @@ void BoardView::drawPieces(QPaintEvent* event)
             continue;
         }
 
+        if(m_dragged != Empty)
+        {
+            Square from = squareAt(m_dragStart);
+            if (from == square)
+            {
+                continue;
+            }
+        }
+
         QPoint pos = posFromSquare(square);
 
         p.drawPixmap(pos, m_theme.piece(m_board.pieceAt(square)));
@@ -342,6 +381,7 @@ Square BoardView::squareAt(const QPoint& p) const
 void BoardView::mousePressEvent(QMouseEvent* event)
 {
     m_dragStart = event->pos();
+    setStoredMove(InvalidSquare,InvalidSquare);
     emit moveStarted();
 }
 
@@ -450,11 +490,6 @@ void BoardView::mouseMoveEvent(QMouseEvent *event)
         return;
     }
 
-    if(event->modifiers() & Qt::ShiftModifier)
-    {
-        return;
-    }
-
     if(m_dragged != Empty)
     {
         QRect old = QRect(m_dragPoint, m_theme.size());
@@ -464,6 +499,11 @@ void BoardView::mouseMoveEvent(QMouseEvent *event)
         return;
     }
 
+    if(event->modifiers() & Qt::ShiftModifier)
+    {
+        return;
+    }
+
     if((event->pos() - m_dragStart).manhattanLength()
             < QApplication::startDragDistance())
     {
@@ -518,8 +558,10 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
         {
             if(s != InvalidSquare)
             {
-                emit clicked(s, button, mapToGlobal(event->pos()), InvalidSquare);
+                Square from = squareAt(m_dragStart);
+                emit clicked(s, button, mapToGlobal(event->pos()), from);
             }
+            m_dragged = Empty;
             emit moveFinished();
             return;
         }
@@ -586,6 +628,7 @@ void BoardView::mouseReleaseEvent(QMouseEvent* event)
             }
         }
     }
+    m_dragged = Empty;
     emit moveFinished();
 }
 
@@ -960,6 +1003,16 @@ void BoardView::drawArrowAnnotation(QPaintEvent* event, QString annotation)
     drawArrow(square1, square2, color);
 }
 
+Piece BoardView::dragged() const
+{
+    return m_dragged;
+}
+
+void BoardView::setDragged(const Piece &dragged)
+{
+    m_dragged = dragged;
+}
+
 bool BoardView::vAlignTop() const
 {
     return m_vAlignTop;
@@ -981,12 +1034,12 @@ void BoardView::setDisabled(bool disabled)
     QWidget::setDisabled(disabled);
 }
 
-void BoardView::setDbIndex(int dbIndex)
+void BoardView::setDbIndex(QObject* dbIndex)
 {
     m_DbIndex = dbIndex;
 }
 
-int BoardView::dbIndex() const
+QObject* BoardView::dbIndex() const
 {
     return m_DbIndex;
 }
