@@ -6,6 +6,12 @@
 #include "ui_tagdetailwidget.h"
 #include "database.h"
 #include "databaseinfo.h"
+#include "tags.h"
+
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
 
 ECOListWidget::ECOListWidget(QWidget *parent) :
     QWidget(parent),
@@ -13,7 +19,7 @@ ECOListWidget::ECOListWidget(QWidget *parent) :
 {
     setObjectName("ECOListWidget");
     ui->setupUi(this);
-    m_filterModel = new QStringListModel(this);
+    m_filterModel = new QStringListModel(ui->tagList);
     ui->tagList->setModel(m_filterModel);
     ui->renameItem->setVisible(false);
 
@@ -85,7 +91,7 @@ void ECOListWidget::ecoSelected(const QString& eco)
         m_eco.setCode(eco);
         ui->filterDatabase->setEnabled(true);
         ui->renameItem->setEnabled(true);
-        QString opName = Game::findEcoName(eco);
+        QString opName = Game::findEcoNameDetailed(eco);
 
         QString head = QString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\"><html><head><title>%1</title><meta name='qrichtext' content='1'><meta http-equiv=\"Content-type\" content=\"text/html;charset=UTF-8\"></head>").arg(eco);
         QString text = QString("%1<body><h1><a href='eco:%2'>%3</a></h1><p>%4</p><p>%5</p><p>%6%7</p>%8</body></html>")
@@ -152,7 +158,7 @@ void ECOListWidget::setDatabase(DatabaseInfo* dbInfo)
     while (iter != m_list.end())
     {
         QString eco = *iter;
-        *iter = eco + " " + Game::findEcoName(eco);
+        *iter = eco + " " + Game::findEcoNameDetailed(eco);
         ++iter;
     }
 
