@@ -10,7 +10,9 @@
 #include "analysiswidget.h"
 #include "boardsetup.h"
 #include "boardview.h"
+#include "chartwidget.h"
 #include "chessbrowser.h"
+#include "clipboarddatabase.h"
 #include "commentdialog.h"
 #include "copydialog.h"
 #include "databaseinfo.h"
@@ -22,6 +24,8 @@
 #include "eventlistwidget.h"
 #include "exclusiveactiongroup.h"
 #include "ficsclient.h"
+#include "ficsconsole.h"
+#include "ficsdatabase.h"
 #include "game.h"
 #include "gamelist.h"
 #include "GameMimeData.h"
@@ -66,7 +70,10 @@
 #include <QTimer>
 #include <QToolBar>
 
-#undef FICS_SUPPORT
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
 
 MainWindow::MainWindow() : QMainWindow(),
     m_tabDragIndex(-1),
@@ -76,15 +83,17 @@ MainWindow::MainWindow() : QMainWindow(),
     m_output(0),
     m_currentFrom(InvalidSquare),
     m_currentTo(InvalidSquare),
+    m_lastColor('G'),
     m_machineHasToMove(false),
-    m_bInDrag(false),
     m_gameMode(false)
 {
     setObjectName("MainWindow");
 
-#ifdef FICS_SUPPORT
-    m_ficsClient = new FicsClient(this);
-#endif
+    // Style::setStyle(this);
+    m_messageTimer = new QTimer(this);
+    m_messageTimer->setInterval(5000);
+    m_messageTimer->setSingleShot(true);
+    connect(m_messageTimer, SIGNAL(timeout()), this, SLOT(slotStatusMessage()));
 
     m_autoPlayTimer = new QTimer(this);
     m_autoPlayTimer->setInterval(AppSettings->getValue("/Board/AutoPlayerInterval").toInt());
@@ -96,21 +105,39 @@ MainWindow::MainWindow() : QMainWindow(),
     connect(m_dragTimer, SIGNAL(timeout()), this, SLOT(slotAutoSwitchTab()));
 
     /* Create clipboard database */
-    DatabaseInfo* pClipDB = new DatabaseInfo(&m_undoGroup);
+    DatabaseInfo* pClipDB = new DatabaseInfo(&m_undoGroup, new ClipboardDatabase);
     connect(pClipDB,SIGNAL(signalRestoreState(Game)), SLOT(slotDbRestoreState(Game)));
-    connect(pClipDB,SIGNAL(signalGameModified()), SLOT(slotGameChanged()));
+    connect(pClipDB,SIGNAL(signalGameModified(bool)), SLOT(slotGameChanged()));
     connect(pClipDB,SIGNAL(signalMoveChanged()), SLOT(slotMoveChanged()));
     connect(pClipDB,SIGNAL(searchProgress(int)), SLOT(slotBoardSearchUpdate(int)), Qt::QueuedConnection);
     connect(pClipDB,SIGNAL(searchFinished()), SLOT(slotBoardSearchFinished()), Qt::QueuedConnection);
+    connect(pClipDB,SIGNAL(signalGameModified(bool)), SIGNAL(signalGameModified(bool)));
     m_databases.append(pClipDB);
     m_currentDatabase = 0;
 
+    /* Create FICS database */
+    FicsDatabase* pDB = new FicsDatabase;
+    pDB->open(QString(),false);
+    DatabaseInfo* pFicsDB = new DatabaseInfo(&m_undoGroup, pDB);
+    pFicsDB->loadGame(0);
+
+    connect(pFicsDB,SIGNAL(signalGameModified(bool)), SLOT(slotGameChanged()));
+    connect(pFicsDB,SIGNAL(signalMoveChanged()), SLOT(slotMoveChanged()));
+    connect(pFicsDB,SIGNAL(signalGameModified(bool)), SIGNAL(signalGameModified(bool)));
+    m_databases.append(pFicsDB);
+
     /* Game List */
     DockWidgetEx* gameListDock = new DockWidgetEx(tr("Game List"), this);
     gameListDock->setObjectName("GameList");
     m_gameList = new GameList(databaseInfo()->filter(), gameListDock);
 
+    // Init before setupActions
+    m_ficsClient = new FicsClient(this);
+    m_ficsConsole = new FicsConsole(this, m_ficsClient);
+
     /* Actions */
+    m_gameModeGroup = new QActionGroup(this);
+    connect(this, SIGNAL(signalGameModeChanged(bool)), m_gameModeGroup, SLOT(setDisabled(bool)));
     setupActions();
 
     /* Delete on close */
@@ -123,15 +150,31 @@ MainWindow::MainWindow() : QMainWindow(),
     m_tabWidget = new QTabWidget(this);
     m_tabWidget->setObjectName("BoardView");
     m_tabWidget->setTabsClosable(true);
-    QToolButton* button = new QToolButton();
-    button->setIcon(QIcon(":/images/new_board.png"));
-    m_tabWidget->setCornerWidget(button);
-    connect(button, SIGNAL(clicked()), SLOT(slotCreateBoardView()));
+    m_tabWidget->setElideMode(Qt::ElideMiddle);
     connect(m_tabWidget, SIGNAL(tabCloseRequested(int)), SLOT(slotCloseBoardView(int)));
-    connect(m_tabWidget, SIGNAL(currentChanged(int)), SLOT(slotActivateBoardView(int)));
+    connect(m_tabWidget, SIGNAL(tabBarClicked(int)), SLOT(slotActivateBoardView(int)));
     /* Board layout */
     m_boardSplitter->addWidget(m_tabWidget);
 
+    DockWidgetEx* ficsConsoleDock = new DockWidgetEx(tr("FICS Console"), this);
+    ficsConsoleDock->setObjectName("FicsCOnsoleDock");
+    ficsConsoleDock->setMinimumSize(150, 100);
+    ficsConsoleDock->setWidget(m_ficsConsole);
+    addDockWidget(Qt::RightDockWidgetArea, ficsConsoleDock);
+    connect(m_ficsConsole, SIGNAL(ReceivedBoard(int,QString)), this, SLOT(HandleFicsBoardRequest(int,QString)));
+    connect(m_ficsConsole, SIGNAL(SignalGameResult(QString)), this, SLOT(HandleFicsResultRequest(QString)));
+    connect(m_ficsConsole, SIGNAL(RequestNewGame()), this, SLOT(HandleFicsNewGameRequest()));
+    connect(m_ficsConsole, SIGNAL(RequestSaveGame()), this, SLOT(HandleFicsSaveGameRequest()));
+    connect(m_ficsConsole, SIGNAL(RequestCloseFICS()), this, SLOT(HandleFicsCloseRequest()));
+    connect(m_ficsConsole, SIGNAL(RequestAddTag(QString,QString)), this, SLOT(HandleFicsAddTagRequest(QString,QString)));
+    connect(m_ficsConsole, SIGNAL(RequestGameMode(bool)), this, SLOT(enterGameMode(bool)));
+    connect(m_ficsConsole, SIGNAL(RequestRemoveLastMove()), this, SLOT(HandleFicsRequestRemoveMove()));
+    connect(m_ficsConsole, SIGNAL(SignalPlayerIsBlack(bool)), this, SLOT(slotFlipView(bool)));
+    connect(m_ficsClient, SIGNAL(disconnected()), SLOT(FicsDisconnected()), Qt::QueuedConnection);
+    connect(m_ficsClient, SIGNAL(connected()), SLOT(FicsConnected()), Qt::QueuedConnection);
+    connect(m_ficsConsole, SIGNAL(RequestStoredMove(Square, Square)), SLOT(slotBoardMove(Square, Square)));
+    m_ficsConsole->setEnabled(false);
+
     /* Game view */
     DockWidgetEx* gameTextDock = new DockWidgetEx(tr("Game Text"), this);
     gameTextDock->setObjectName("GameTextDock");
@@ -155,9 +198,11 @@ MainWindow::MainWindow() : QMainWindow(),
         annotatedTime->display("1:00:00");
         if(i == 0)
         {
-            QWidget* spacer = new QWidget();
-            spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-            m_gameToolBar->addWidget(spacer);
+            ChartWidget* chartWidget = new ChartWidget(m_gameToolBar);
+            chartWidget->setObjectName("ChartWidget");
+            chartWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+            connect(chartWidget, SIGNAL(halfMoveRequested(int)), this, SLOT(slotGameMoveToPly(int)));
+            m_gameToolBar->addWidget(chartWidget);
         }
     }
 
@@ -272,10 +317,15 @@ MainWindow::MainWindow() : QMainWindow(),
     connect(this, SIGNAL(reconfigure()), m_databaseList, SLOT(slotReconfigure()));
     connect(m_databaseList, SIGNAL(requestMakeBook(QString)),
             this, SLOT(slotMakeBook(QString)));
-    m_databaseList->addFileOpen(QString(), false);
-    m_databaseList->setFileCurrent(QString());
+
+    m_databaseList->addFileOpen(pClipDB->database()->name(), false);
+    m_databaseList->addFileOpen(pFicsDB->database()->name(), false);
+    m_databaseList->setFileClose(pFicsDB->database()->name(), 0);
+    m_databaseList->setFileCurrent(pClipDB->database()->name());
+
     restoreRecentFiles();
     connect(m_databaseList, SIGNAL(raiseRequest()), dbListDock, SLOT(raise()));
+    connect(this, SIGNAL(signalGameModeChanged(bool)), m_databaseList, SLOT(setDisabled(bool)));
 
     /* Recent files */
     m_recentFiles.restore();
@@ -291,12 +341,13 @@ MainWindow::MainWindow() : QMainWindow(),
     m_menuView->addAction(openingDock->toggleViewAction());
     connect(openingDock->toggleViewAction(), SIGNAL(triggered()), SLOT(slotSearchTree()));
     connect(openingDock, SIGNAL(visibilityChanged(bool)), m_openingTreeWidget, SLOT(cancel()));
-    connect(m_openingTreeWidget, SIGNAL(signalTreeUpdated()), this, SLOT(slotTreeUpdate()));
+    connect(m_openingTreeWidget, SIGNAL(signalTreeUpdated(bool)), this, SLOT(slotTreeUpdate(bool)));
     connect(m_openingTreeWidget, SIGNAL(signalSourceChanged()), this, SLOT(slotSearchTree()));
     connect(this, SIGNAL(reconfigure()), m_openingTreeWidget, SLOT(slotReconfigure()));
     openingDock->toggleViewAction()->setShortcut(Qt::CTRL + Qt::Key_T);
     openingDock->hide();
     connect(this, SIGNAL(signalDatabaseOpenClose()), this, SLOT(slotUpdateOpeningTreeWidget()));
+    connect(this, SIGNAL(signalGameModeChanged(bool)), m_openingTreeWidget, SLOT(setDisabled(bool)));
 
     /* Analysis Dock */
     DockWidgetEx* analysisDock = new DockWidgetEx(tr("Analysis 1"), this);
@@ -318,6 +369,7 @@ MainWindow::MainWindow() : QMainWindow(),
     analysisDock->toggleViewAction()->setShortcut(Qt::CTRL + Qt::Key_F2);
     analysisDock->hide();
     connect(this, SIGNAL(signalGameLoaded()), analysis, SLOT(slotUciNewGame()));
+    connect(this, SIGNAL(signalGameModeChanged(bool)), analysis, SLOT(setDisabled(bool)));
 
     m_mainAnalysis = analysis;
 
@@ -341,10 +393,14 @@ MainWindow::MainWindow() : QMainWindow(),
     analysisDock2->toggleViewAction()->setShortcut(Qt::CTRL + Qt::Key_F3);
     analysisDock2->hide();
     connect(this, SIGNAL(signalGameLoaded()), analysis, SLOT(slotUciNewGame()));
+    connect(this, SIGNAL(signalGameModeChanged(bool)), analysis, SLOT(setDisabled(bool)));
 
     /* Randomize */
     srand(time(0));
 
+    /* Append the FICS console to the view menu */
+    m_menuView->addAction(ficsConsoleDock->toggleViewAction());
+
     /* Restoring layouts */
     if(!AppSettings->layout(this))
     {
@@ -395,16 +451,17 @@ MainWindow::MainWindow() : QMainWindow(),
     statusBar()->setSizeGripEnabled(true);
     m_progressBar = new QProgressBar();
 
-    /* Reconfigure. */
-    slotReconfigure();
-
     /* Very late as this will update other widgets */
     connect(this, SIGNAL(databaseModified()), SLOT(slotDatabaseModified()));
     connect(this, SIGNAL(signalDatabaseOpenClose()), this, SLOT(updateMenuDatabases()));
     CreateBoardView();
 
+    /* Setup the dimensions of all widgets and the main board */
+    slotReconfigure();
+
     /* Display main window */
     show();
+    downloadManager = new DownloadManager(this);
 
     /* Load files from command line */
     QStringList args = qApp->arguments();
@@ -412,7 +469,7 @@ MainWindow::MainWindow() : QMainWindow(),
     {
         if(QFile::exists(args[i]))
         {
-            openDatabaseUrl(args[i], false);
+            openDatabaseArchive(args[i], false);
         }
     }
 
@@ -425,7 +482,7 @@ MainWindow::MainWindow() : QMainWindow(),
     setAcceptDrops(true);
 
     QString dir = AppSettings->commonDataPath();
-    QDir().mkpath(dir + "/index");
+    QDir().mkpath(dir + QDir::separator() + "index");
 
     /* Load ECO file */
     slotStatusMessage(tr("Loading ECO file..."));
@@ -433,7 +490,6 @@ MainWindow::MainWindow() : QMainWindow(),
     connect(ecothread, SIGNAL(loaded(QObject*, bool)), this, SLOT(ecoLoaded(QObject*, bool)));
     ecothread->start();
     StartCheckUpdate();
-    downloadManager = new DownloadManager(this);
 }
 
 MainWindow::~MainWindow()
@@ -441,8 +497,12 @@ MainWindow::~MainWindow()
     m_autoPlayTimer->stop();
     m_dragTimer->stop();
     m_openingTreeWidget->cancel();
+    m_ficsConsole->Terminate();
+
     foreach(DatabaseInfo * database, m_databases)
     {
+        // Avoid any GUI action if a database is closed
+        disconnect(database,SIGNAL(signalGameModified(bool)), this, SLOT(slotGameChanged()));
         database->close();
     }
     qDeleteAll(m_databases.begin(), m_databases.end());
@@ -467,7 +527,7 @@ bool MainWindow::eventFilter(QObject *obj, QEvent *event)
                             keyEvent->key() == Qt::Key_Return ||
                             keyEvent->key() == Qt::Key_Enter))
             {
-                if (obj == this || obj == m_boardView || obj == m_gameView)
+                if (obj == this || obj == m_boardView || obj == m_gameView || obj == m_mainAnalysis)
                 {
                     keyPressEvent(keyEvent);
                     return true;
@@ -563,7 +623,14 @@ void MainWindow::evaluateSanNag(QKeyEvent *e)
             m_nagText.clear();
             return;
         }
-        addVariation(m_nagText);
+        if (addVariation(m_nagText))
+        {
+            if (database()->name() == "FICS")
+            {
+                Move m = game().move();
+                m_ficsConsole->SendMove(m.toAlgebraic());
+            }
+        }
         m_nagText.clear();
         return;
     }
@@ -572,6 +639,11 @@ void MainWindow::evaluateSanNag(QKeyEvent *e)
         m_nagText.append(e->text());
         if (addVariation(m_nagText))
         {
+            if (database()->name() == "FICS")
+            {
+                Move m = game().move();
+                m_ficsConsole->SendMove(m.toAlgebraic());
+            }
             m_nagText.clear();
             return;
         }
@@ -617,17 +689,18 @@ Database* MainWindow::database()
     return databaseInfo()->database();
 }
 
+const Database* MainWindow::database() const
+{
+    return databaseInfo()->database();
+}
+
 QString MainWindow::databaseName(int index) const
 {
     if(index < 0)
     {
         index = m_currentDatabase;
     }
-    QString name = m_databases[index]->database()->name();
-    if(name.isEmpty())
-    {
-        return tr("[Clipboard]");
-    }
+    QString name = m_databases[index]->database() ? m_databases[index]->database()->name() : "";
     return name;
 }
 
@@ -679,9 +752,14 @@ void MainWindow::gameLoad(int index)
         {
             m_gameList->selectGame(index);
             emit signalGameIsEmpty(true);
+            UpdateBoardInformation();
             slotGameChanged();
             m_gameList->setFocus();
             emit signalGameLoaded();
+            if(m_training->isChecked())
+            {
+                m_boardView->setFlipped((game().board().toMove() == Black));
+            }
         }
     }
 }
@@ -787,16 +865,40 @@ void MainWindow::openDatabase(QString fname)
 
 void MainWindow::openDatabaseUrl(QString fname, bool utf8)
 {
+    QFileInfo fi(fname);
     QUrl url = QUrl::fromUserInput(fname);
-    if((url.scheme() == "http") || (url.scheme() == "https") || (url.scheme() == "ftp"))
+    if (!fi.completeSuffix().isEmpty())
     {
-        connect(downloadManager, SIGNAL(downloadError(QUrl)), this, SLOT(loadError(QUrl)));
-        connect(downloadManager, SIGNAL(onDownloadFinished(QUrl, QString)), this, SLOT(loadReady(QUrl, QString)));
-        downloadManager->doDownload(url);
-        return;
+        if (url.isValid() && !url.isRelative() &&
+            ((url.scheme() == "http") || (url.scheme() == "https") || (url.scheme() == "ftp")))
+        {
+            connect(downloadManager, SIGNAL(downloadError(QUrl)), this, SLOT(loadError(QUrl)));
+            connect(downloadManager, SIGNAL(onDownloadFinished(QUrl, QString)), this, SLOT(loadReady(QUrl, QString)));
+            downloadManager->doDownload(url);
+        }
+        else
+        {
+            openDatabaseArchive(url.toLocalFile(), utf8);
+        }
     }
+    else if (fname == "FICS")
+    {
+        openFICS();
+    }
+    else if (fname == "Clipboard")
+    {
+        ActivateDatabase("Clipboard");
+    }
+}
 
-    openDatabaseArchive(url.toLocalFile(), utf8);
+void MainWindow::openFICS()
+{
+    if (!m_ficsClient->sessionStarted())
+    {
+        m_ficsClient->startSession();
+        m_databaseList->addFileOpen("FICS", false);
+    }
+    ActivateDatabase("FICS");
 }
 
 void MainWindow::openDatabaseArchive(QString fname, bool utf8)
@@ -825,7 +927,7 @@ void MainWindow::openDatabaseArchive(QString fname, bool utf8)
                 for(bool more = zip.goToFirstFile(); more; more = zip.goToNextFile())
                 {
                     file.open(QIODevice::ReadOnly);
-                    QString outName = dir + "/" + file.getActualFileName();
+                    QString outName = dir + QDir::separator() + file.getActualFileName();
                     QDir pathOut;
                     outName = pathOut.absoluteFilePath(outName);
                     if(!QFile::exists(outName))
@@ -857,31 +959,50 @@ void MainWindow::openDatabaseArchive(QString fname, bool utf8)
     }
 }
 
-void MainWindow::openDatabaseFile(QString fname, bool utf8)
+bool MainWindow::ActivateDatabase(QString fname)
 {
-    QFileInfo fi = QFileInfo(fname);
-    fname = fi.canonicalFilePath();
-
-    /* Check if the database isn't already open */
+    /* Check if the database is open */
     for(int i = 0; i < m_databases.count(); i++)
     {
         if(m_databases[i]->database()->filename() == fname)
         {
             if(m_databases[i]->isValid())
             {
-                m_currentDatabase = i;
-                m_boardView->setDbIndex(m_currentDatabase);
-                UpdateBoardInformation();
-                m_databaseList->setFileCurrent(fname);
-                slotDatabaseChanged();
+                if (!m_databases[i]->IsBook())
+                {
+                    if (i != m_currentDatabase)
+                    {
+                        m_currentDatabase = i;
+                        m_databaseList->setFileCurrent(fname);
+                        slotDatabaseChanged();
+                    }
+                    activateBoardViewForDbIndex(m_databases[m_currentDatabase]);
+                }
             }
             else
             {
-                slotStatusMessage(tr("Database %1 cannot be accessed at the moment.").arg(fi.fileName()));
+                slotStatusMessage(tr("Database %1 cannot be accessed at the moment.").arg(fname));
             }
-            return;
+            return true;
         }
     }
+    return false;
+}
+
+void MainWindow::openDatabaseFile(QString fname, bool utf8)
+{
+    QFileInfo fi = QFileInfo(fname);
+    fname = fi.canonicalFilePath();
+    if (fname.isEmpty())
+    {
+        cancelOperation("File not found.");
+        return;
+    }
+    if (ActivateDatabase(fname))
+    {
+        /* Check if the database is already open */
+        return;
+    }
 
     // Create database, connect progress bar and open file
     DatabaseInfo* db = new DatabaseInfo(&m_undoGroup,fname);
@@ -891,10 +1012,11 @@ void MainWindow::openDatabaseFile(QString fname, bool utf8)
     connect(db->database(), SIGNAL(progress(int)), SLOT(slotOperationProgress(int)));
     connect(db, SIGNAL(LoadFinished(DatabaseInfo*)), this, SLOT(slotDataBaseLoaded(DatabaseInfo*)));
     connect(db, SIGNAL(signalRestoreState(Game)), SLOT(slotDbRestoreState(Game)));
-    connect(db, SIGNAL(signalGameModified()), SLOT(slotGameChanged()));
+    connect(db, SIGNAL(signalGameModified(bool)), SLOT(slotGameChanged()));
     connect(db, SIGNAL(signalMoveChanged()), SLOT(slotMoveChanged()));
     connect(db, SIGNAL(searchProgress(int)), SLOT(slotBoardSearchUpdate(int)), Qt::QueuedConnection);
     connect(db, SIGNAL(searchFinished()), SLOT(slotBoardSearchFinished()), Qt::QueuedConnection);
+    connect(db, SIGNAL(signalGameModified(bool)), SIGNAL(signalGameModified(bool)));
     if(!db->open(utf8))
     {
         slotDataBaseLoaded(db);
@@ -902,7 +1024,7 @@ void MainWindow::openDatabaseFile(QString fname, bool utf8)
     else
     {
         m_databases.append(db);
-    }
+    }    
 }
 
 void MainWindow::loadError(QUrl url)
@@ -937,9 +1059,11 @@ void MainWindow::slotDataBaseLoaded(DatabaseInfo* db)
     {
         if(m_databases[i]->database()->filename() == fname)
         {
-            m_currentDatabase = i;
-            m_boardView->setDbIndex(m_currentDatabase);
-            UpdateBoardInformation();
+            if (!m_databases[i]->IsBook())
+            {
+                m_currentDatabase = i;
+                CreateBoardView();
+            }
         }
     }
     m_databaseList->setFileCurrent(fname);
@@ -1105,6 +1229,7 @@ void MainWindow::setupActions()
     file->addAction(createAction(tr("&New database..."), SLOT(slotFileNew()), QKeySequence(), fileToolBar, ":/images/new.png"));
     file->addAction(createAction(tr("&Open..."), SLOT(slotFileOpen()), QKeySequence::Open, fileToolBar, ":/images/folder_open.png"));
     file->addAction(createAction(tr("Open in UTF8..."), SLOT(slotFileOpenUtf8()), QKeySequence()));
+    file->addAction(createAction(tr("Open FICS"), SLOT(openFICS()), QKeySequence()));
     QMenu* menuRecent = file->addMenu(tr("Open &recent..."));
 
     for(int i = 0; i < MaxRecentFiles; ++i)
@@ -1117,8 +1242,10 @@ void MainWindow::setupActions()
     }
     file->addSeparator();
 
-    QAction* commitAction = createAction(tr("&Save"), SLOT(slotFileSave()), Qt::CTRL + Qt::SHIFT + Qt::Key_S, fileToolBar, ":/images/save.png");
+    QAction* commitAction = createAction(tr("Save Database"), SLOT(slotFileSave()), Qt::CTRL + Qt::SHIFT + Qt::Key_S, fileToolBar, ":/images/save.png");
+#ifndef QT_NO_TOOLTIP
     commitAction->setToolTip(tr("Commit Database to disk"));
+#endif // QT_NO_TOOLTIP
     connect(this, SIGNAL(signalCurrentDBisReadWrite(bool)), commitAction, SLOT(setEnabled(bool)));
     file->addAction(commitAction);
 
@@ -1130,6 +1257,8 @@ void MainWindow::setupActions()
 
     QAction* saveImage = createAction(tr("Export Image"), SLOT(slotExportImage()));
     file->addAction(saveImage);
+    file->addAction(createAction(tr("Mail PGN..."), SLOT(slotSendMail()), QKeySequence(), fileToolBar, ":/images/edit_mail.png"));
+
     file->addSeparator();
 
     QAction* closeFileAction = createAction(tr("&Close"), SLOT(slotFileClose()), QKeySequence::Close, fileToolBar, ":/images/folder.png");
@@ -1170,13 +1299,17 @@ void MainWindow::setupActions()
     QMenu* editVariation = edit->addMenu(tr("Variation"));
 
     QAction* promoteAction = createAction(editVariation, tr("Promote"), SLOT(slotEditVarPromote()), Qt::CTRL + Qt::Key_J, editToolBar, ":/images/format_indent_less.png");
+#ifndef QT_NO_TOOLTIP
     promoteAction->setToolTip(tr("Promote Variation"));
+#endif // QT_NO_TOOLTIP
     connect(this, SIGNAL(signalMoveHasParent(bool)), promoteAction, SLOT(setEnabled(bool)));
     editVariation->addAction(promoteAction);
 
     QAction* removeVariationAction = createAction(editVariation, tr("Remove"), SLOT(slotEditVarRemove()),
                                      Qt::CTRL + Qt::Key_Delete, editToolBar, ":/images/edit_cut.png");
+#ifndef QT_NO_TOOLTIP
     removeVariationAction->setToolTip(tr("Remove Variation"));
+#endif // QT_NO_TOOLTIP
     connect(this, SIGNAL(signalMoveHasParent(bool)), removeVariationAction, SLOT(setEnabled(bool)));
     editVariation->addAction(removeVariationAction);
 
@@ -1212,7 +1345,7 @@ void MainWindow::setupActions()
     QMenu* toolbars = m_menuView->addMenu(tr("Toolbars"));
     m_menuView->addSeparator();
 
-#if defined(Q_OS_WIN)
+#if defined(Q_OS_WINXX)
     QAction* stayOnTop = createAction(tr("Stay on Top"), SLOT(slotToggleStayOnTop()));
     stayOnTop->setCheckable(true);
     stayOnTop->setChecked(AppSettings->getValue("/MainWindow/StayOnTop").toBool());
@@ -1222,8 +1355,6 @@ void MainWindow::setupActions()
     AppSettings->setValue("/MainWindow/StayOnTop", false);
 #endif
 
-    m_menuView->addAction(createAction(tr("New board"), SLOT(slotCreateBoardView()), Qt::CTRL + Qt::SHIFT + Qt::Key_N,
-                                       0, QIcon(":/images/new_board.png")));
     m_menuView->addAction(createAction(tr("Close current board"), SLOT(slotCloseBoardView()), Qt::CTRL + Qt::SHIFT + Qt::Key_W,
                                        0, style()->standardIcon(QStyle::SP_TitleBarCloseButton)));
     m_menuView->addSeparator();
@@ -1253,25 +1384,32 @@ void MainWindow::setupActions()
     loadMenu->addAction(prevAction);
     loadMenu->addAction(createAction(tr("&Go to game..."), SLOT(slotGameLoadChosen()), Qt::CTRL + Qt::Key_G));
     loadMenu->addAction(createAction(tr("&Random"), SLOT(slotGameLoadRandom()), Qt::CTRL + Qt::Key_Question));
-    gameMenu->addAction(createAction(tr("&Save..."), SLOT(slotGameSave()), QKeySequence::Save));
+    QAction* saveAction = createAction(tr("&Save..."), SLOT(slotGameSave()), QKeySequence::Save);
+    gameMenu->addAction(saveAction);
+    QAction* saveOnlyAction = createAction(tr("Save"), SLOT(slotGameSaveOnly()));
+    connect(this, SIGNAL(signalGameModified(bool)), saveOnlyAction, SLOT(setEnabled(bool)));
+    gameMenu->addAction(saveOnlyAction);
+
     gameMenu->addAction(createAction(tr("Edit tags..."), SLOT(slotGameEditTags())));
 
     gameMenu->addSeparator();
 
     QAction* flip = createAction(tr("&Flip board"), SLOT(slotConfigureFlip()), Qt::CTRL + Qt::Key_B, gameToolBar, ":/images/flip_board.png");
     flip->setCheckable(true);
+    connect(m_ficsConsole, SIGNAL(SignalPlayerIsBlack(bool)), flip, SLOT(setChecked(bool)));
     gameToolBar->addSeparator();
 
-    ExclusiveActionGroup* autoGroup = new ExclusiveActionGroup(this);
+    autoGroup = new ExclusiveActionGroup(this);
     ExclusiveActionGroup* autoGroup2 = new ExclusiveActionGroup(this);
 
-    m_match = createAction(tr("Match"), SLOT(slotToggleGameMode()), Qt::CTRL + Qt::Key_M, gameToolBar, ":/images/black_chess.png");
-    m_match->setCheckable(true);
-    autoGroup2->addAction(m_match);
-    autoGroup->addAction(m_match);
-    gameMenu->addAction(m_match );
+    QAction* match = createAction(tr("Match"), SLOT(slotToggleGameMode()), Qt::CTRL + Qt::Key_M, gameToolBar, ":/images/black_chess.png");
+    match->setCheckable(true);
+
+    autoGroup2->addAction(match);
+    autoGroup->addAction(match);
+    gameMenu->addAction(match);
     gameMenu->addSeparator();
-    connect(m_match, SIGNAL(changed()), SLOT(slotToggleGameMode()));
+    connect(match, SIGNAL(changed()), SLOT(slotToggleGameMode()));
 
     m_training = createAction(tr("Training"), SLOT(slotToggleTraining()), Qt::CTRL + Qt::Key_R, gameToolBar, ":/images/training.png");
     m_training->setCheckable(true);
@@ -1303,7 +1441,9 @@ void MainWindow::setupActions()
     /* Game->Go to submenu */
     QMenu* goMenu = gameMenu->addMenu(tr("&Go to"));
     QAction* gotoFirstMove = createAction(goMenu, tr("&Start"), SLOT(slotGameMoveFirst()), Qt::Key_Home, gameToolBar, ":/images/first.png");
+#ifndef QT_NO_TOOLTIP
     gotoFirstMove->setToolTip(tr("Go to first move"));
+#endif // QT_NO_TOOLTIP
     connect(this, SIGNAL(signalMoveHasPreviousMove(bool)), gotoFirstMove, SLOT(setEnabled(bool)));
     goMenu->addAction(gotoFirstMove);
 
@@ -1316,7 +1456,9 @@ void MainWindow::setupActions()
     goMenu->addAction(actionNextMove);
 
     QAction* gotoLastMove = createAction(goMenu, tr("&End"), SLOT(slotGameMoveLast()), Qt::Key_End, gameToolBar, ":/images/last.png");
+#ifndef QT_NO_TOOLTIP
     gotoLastMove->setToolTip(tr("Go to last move"));
+#endif // QT_NO_TOOLTIP
     connect(this, SIGNAL(signalMoveHasNextMove(bool)), gotoLastMove, SLOT(setEnabled(bool)));
     goMenu->addAction(gotoLastMove);
 
@@ -1399,7 +1541,8 @@ void MainWindow::setupActions()
 
     help->addAction(createAction(tr("Customize Keyboard..."), SLOT(slotEditActions())));
     help->addSeparator();
-    QAction* reportBugAction = createAction(tr("&Report a bug..."), SLOT(slotHelpBug()));
+    help->addAction(createAction(tr("Load Sample Database"), SLOT(QueryLoadDatabase())));
+    QAction* reportBugAction = createAction(tr("Report a bug..."), SLOT(slotHelpBug()));
     reportBugAction->setIcon(QIcon(":/images/bug.png"));
     help->addAction(reportBugAction);
     help->addSeparator();
@@ -1426,14 +1569,14 @@ void MainWindow::setupActions()
 bool MainWindow::confirmQuit()
 {
     QString modified;
-    for(int i = 1; i < m_databases.size(); i++)
+    for(int i = 2; i < m_databases.size(); i++)
     {
         if (!QuerySaveGame(m_databases[i]))
         {
             return false;
         }
     }
-    for(int i = 1; i < m_databases.size(); i++)
+    for(int i = 2; i < m_databases.size(); i++)
     {
         if(m_databases[i]->isValid() && m_databases[i]->database()->isModified())
         {
@@ -1451,13 +1594,13 @@ bool MainWindow::confirmQuit()
         if(response == MessageDialog::Yes)
         {
             Output output(Output::Pgn);
-            for(int i = 1; i < m_databases.size(); i++)
+            for(int i = 2; i < m_databases.size(); i++)
                 if(m_databases[i]->database()->isModified())
                     output.output(m_databases[i]->database()->filename(),
                                   *(m_databases[i]->database()));
         }
     }
-    for(int i = 1; i < m_databases.size(); i++)
+    for(int i = 2; i < m_databases.size(); i++)
     {
         slotFileCloseIndex(i);
     }
@@ -1517,6 +1660,16 @@ bool MainWindow::QuerySaveGame(DatabaseInfo *dbInfo)
     return true;
 }
 
+void MainWindow::SimpleSaveGame()
+{
+    DatabaseInfo *dbInfo = databaseInfo();
+    if(dbInfo && dbInfo->gameNeedsSaving())
+    {
+        saveGame(dbInfo);
+        emit databaseModified();
+    }
+}
+
 void MainWindow::restoreRecentFiles()
 {
     AppSettings->beginGroup("Favorites");
@@ -1577,24 +1730,29 @@ void MainWindow::slotHttpDone(QNetworkReply *reply)
     }
 }
 
+void MainWindow::QueryLoadDatabase()
+{
+    LoadQuery dlg(this);
+    if(dlg.exec() == QDialog::Accepted)
+    {
+        slotStatusMessage(tr("Start loading database..."));
+        if(dlg.largeDB())
+        {
+            openDatabaseUrl("http://chessx.sourceforge.net/db/bundesliga2000.pgn.zip", false);
+        }
+        else
+        {
+            openDatabaseUrl("http://chessx.sourceforge.net/db/SBL1213.pgn.zip", false);
+        }
+    }
+    AppSettings->setValue("/General/BuiltinDbInstalled", QVariant(true));
+}
+
 void MainWindow::StartCheckDatabase()
 {
     if ((m_recentFiles.count() == 0) && !AppSettings->getValue("/General/BuiltinDbInstalled").toBool())
     {
-        LoadQuery dlg(this);
-        if(dlg.exec() == QDialog::Accepted)
-        {
-            slotStatusMessage(tr("Start loading database..."));
-            if(dlg.largeDB())
-            {
-                openDatabaseUrl("http://chessx.sourceforge.net/db/bundesliga2000.pgn.zip", false);
-            }
-            else
-            {
-                openDatabaseUrl("http://chessx.sourceforge.net/db/SBL1213.pgn.zip", false);
-            }
-        }
-        AppSettings->setValue("/General/BuiltinDbInstalled", QVariant(true));
+        QueryLoadDatabase();
     }
 }
 
@@ -1680,4 +1838,5 @@ bool MainWindow::gameMode() const
 void MainWindow::setGameMode(bool gameMode)
 {
     m_gameMode = gameMode;
+    emit signalGameModeChanged(m_gameMode);
 }
