@@ -10,7 +10,6 @@
 #ifndef __MAINWINDOW_H__
 #define __MAINWINDOW_H__
 
-#include "common.h"
 #include "historylist.h"
 #include "output.h"
 
@@ -32,7 +31,9 @@ class ECOListWidget;
 class EcoThread;
 class EditAction;
 class EventListWidget;
+class ExclusiveActionGroup;
 class FicsClient;
+class FicsConsole;
 class Game;
 class GameList;
 class OpeningTreeWidget;
@@ -78,6 +79,8 @@ protected:
     void evaluateSanNag(QKeyEvent *e);
     /** @return active database */
     Database* database();
+    /** @return active database */
+    const Database* database() const;
     /** @return active database structure */
     DatabaseInfo* databaseInfo();
     /** @return database name of database @p index (or active, if no parameter). */
@@ -104,6 +107,8 @@ protected:
     void updateMenuRecent();
     /** Get current mode */
     bool gameMode() const;
+    /** Is premoving available */
+    bool premoveAllowed() const;
     /** Set current Mode */
     void setGameMode(bool gameMode);
     /** Add a variation from keyboard */
@@ -111,6 +116,8 @@ protected:
 public slots:
     /** Enter gaming mode */
     void slotToggleGameMode();
+    /** Flip the board view */
+    void slotFlipView(bool);
     void enterGameMode(bool gameMode);
     /** Open database */
     void openDatabase(QString fname);
@@ -150,6 +157,8 @@ public slots:
     void slotEditTruncateEnd();
     /** Truncate moves from the beginning of the game. */
     void slotEditTruncateStart();
+    /** Mail position's PGN. */
+    void slotSendMail();
     /**  */
     void slotReconfigure();
     /**  */
@@ -217,6 +226,9 @@ public slots:
     /** Save game, replacing old one if it is edited, appending if it is new
         @return false if the saving was cancelled, true if changes are to be saved or discarded. */
     bool slotGameSave();
+    /** Save game, replacing old one if it is edited, appending if it is new
+        @return false if the saving was cancelled, true if changes are to be saved or discarded. */
+    void slotGameSaveOnly();
     /** Start a dialog which edits the games tags */
     void slotGameEditTags();
     /** Add variation to current position. */
@@ -281,13 +293,13 @@ public slots:
     /** Reverse current filter */
     void slotSearchReverse();
     /** Make an Update of the game list after the opening tree was updated */
-    void slotTreeUpdate();
+    void slotTreeUpdate(bool dbIsFilterSource);
     /** Show opening tree */
     void slotSearchTree();
     /** Move @p index was selected in Opening Tree. */
     void slotSearchTreeMove(const QModelIndex& index);
     /** Made given move on the board */
-    void slotBoardMove(Square from, Square to, int button);
+    void slotBoardMove(Square from, Square to, int button=0);
     /** Move started - use together with moveFinished to avoid automatic moves during user interaction */
     void slotMoveStarted();
     /** Move finished - use together with moveStarted to avoid automatic moves during user interaction */
@@ -299,8 +311,10 @@ public slots:
     /** Update GUI after current move was changed. Update BoardView to current board,
     updates GameInfo and highlight current move in GameView. */
     void slotMoveChanged();
+    /** Move to a given move (@p ply) */
+    void slotGameMoveToPly(int ply);
     /** Show temporary message. */
-    void slotStatusMessage(const QString& msg);
+    void slotStatusMessage(const QString& msg = QString());
     /** Show progress bar for open file. */
     void slotOperationProgress(int progress);
     /** Change database. */
@@ -343,6 +357,18 @@ public slots:
     void slotBookBuildError(QString path);
     /** Merge the clipboard into the current game */
     void slotEditMergePGN();
+    /** Handle a new board from a FICS request */
+    void HandleFicsBoardRequest(int, QString);
+    /** Enter a result from a fics game */
+    void HandleFicsResultRequest(QString s);
+    /** EStart a new game from FICS */
+    void HandleFicsNewGameRequest();
+    /** Save the current game from FICS */
+    void HandleFicsSaveGameRequest();
+    /** Add a tags value to a game from FICS */
+    void HandleFicsAddTagRequest(QString tag,QString value);
+    /** Close FICS upon request from the console */
+    void HandleFicsCloseRequest();
 protected slots:
     /** Update recent files menu */
     void updateMenuDatabases();
@@ -368,8 +394,6 @@ protected slots:
     void slotRedArrowHere();
     /** Set the Stay On Top Property */
     void slotToggleStayOnTop();
-    /** Create a new, empty Board document */
-    void slotCreateBoardView();
     /** Activate an existing board document */
     void slotActivateBoardView(int);
     /** Activate an existing board document */
@@ -386,14 +410,20 @@ protected slots:
     void slotMoveIntervalChanged(int);
     /** Value for Move Interval Slider is changed */
     void slotSetSliderText(int);
-
+    /** Query for a database to load */
+    void QueryLoadDatabase();
+    /** Handle a disconnection from fics */
+    void FicsDisconnected();
+    /** Handle event that a FICS session was opened */
+    void FicsConnected();
+    void HandleFicsRequestRemoveMove();
+    void openFICS();
 protected:
     void moveChanged();
     bool pasteFen(QString& errorText, QString fen, bool newGame=false);
     void copyGame(int target, int index);
     Database* getDatabaseByPath(QString path);
     DatabaseInfo* getDatabaseInfoByPath(QString path);
-    void activateBoardView(int n);
     void updateOpeningTree(const Board& b, bool atEnd);
 
 protected:
@@ -405,6 +435,15 @@ protected:
     QTabBar* m_pDragTabBar;
     QTimer* m_dragTimer;
 
+    void activateBoardView(int n);
+    void activateBoardViewForDbIndex(void *dbIndex);
+    void closeBoardViewForDbIndex(void *dbIndex);
+    int findBoardView(void *dbIndex) const;
+    void UpdateMaterial();
+    bool ActivateDatabase(QString fname);
+    bool addRemoteMoveFrom64Char(QString s);
+    void newGame();
+    void AutoMoveAtEndOfGame();
 signals:
     /** Re-read configuration. */
     void reconfigure();
@@ -425,6 +464,10 @@ signals:
     void signalGameIsEmpty(bool);
     void signalGameAtLineStart(bool);
 
+    void signalGameModeChanged(bool);
+
+    void signalGameModified(bool gameNeedsSaving);
+
     void signalCurrentDBisReadWrite(bool);
     void signalCurrentDBcanBeClosed(bool);
     void signalCurrentDBhasGames(bool);
@@ -472,10 +515,12 @@ private:
     /** Query User and save game if game was modified
         @return true if the next action shall be performed */
     bool QuerySaveGame(DatabaseInfo* dbInfo = 0);
+    /** Save game without query into current database */
+    void SimpleSaveGame();
     /** Save game without query */
     void saveGame(DatabaseInfo *dbInfo);
     /** Load next game without query */
-    void loadNextGame();
+    bool loadNextGame();
     /** Save Database without query or progress bar */
     void saveDatabase(DatabaseInfo *dbInfo);
     /** Save Database with query */
@@ -532,13 +577,15 @@ private:
     /** Currently updated tree. May be NULL if no update in progress. */
     QString m_nagText;
     QTimer* m_autoPlayTimer;
+    QTimer* m_messageTimer;
     int m_currentFrom;
     int m_currentTo;
     AnalysisWidget* m_mainAnalysis;
     Board m_AutoInsertLastBoard;
     Square m_annotationSquare;
     Square m_annotationSquareFrom;
-    QAction* m_match;
+    QChar m_lastColor;
+    ExclusiveActionGroup* autoGroup;
     QAction* m_training;
     QAction* m_autoRespond;
     QAction* m_autoPlay;
@@ -547,9 +594,11 @@ private:
     QNetworkAccessManager* m_manager;
     DownloadManager* downloadManager;
     bool m_machineHasToMove;
-    bool m_bInDrag;
     bool m_gameMode;
     FicsClient* m_ficsClient;
+    FicsConsole* m_ficsConsole;
+    QActionGroup* m_gameModeGroup;
+
 };
 
 
