@@ -6,19 +6,40 @@
 #include "ui_openingtreewidget.h"
 
 #include <QModelIndex>
+#include <QToolButton>
+#include <QUndoGroup>
+#include <QUndoStack>
 
 #include "boardview.h"
+#include "databaseinfo.h"
 #include "openingtree.h"
 #include "settings.h"
 
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
+
 OpeningTreeWidget::OpeningTreeWidget(QWidget *parent) :
     QWidget(parent),
     ui(new Ui::OpeningTreeWidget)
 {
     ui->setupUi(this);
 
-    m_openingTree = new OpeningTree(this);
-    
+    qRegisterMetaType<QList<MoveData> >("QList<MoveData>");
+
+    m_openingTree = new OpeningTree(ui->OpeningTreeView);
+
+    QUndoGroup* undoGroup = new QUndoGroup(this);
+    m_UndoStack = new QUndoStack(undoGroup);
+    m_UndoStack->setUndoLimit(10);
+    undoGroup->addStack(m_UndoStack);
+    undoGroup->setActiveStack(m_UndoStack);
+
+    connect(ui->btUndo, SIGNAL(clicked()), undoGroup, SLOT(undo()));
+    connect(undoGroup, SIGNAL(canUndoChanged(bool)), ui->btUndo, SLOT(setEnabled(bool)));
+    ui->btUndo->setEnabled(false);
+
     ui->OpeningTreeView->setObjectName("OpeningTree");
     ui->OpeningTreeView->setSortingEnabled(true);
     ui->OpeningTreeView->setModel(m_openingTree);
@@ -59,10 +80,19 @@ Board OpeningTreeWidget::board() const
     return m_openingTree->board();
 }
 
-bool OpeningTreeWidget::updateFilter(Filter& f, const Board& b, bool bEnd)
+void OpeningTreeWidget::updateFilter(Filter& f, const Board& b, bool bEnd)
+{
+    if (m_openingTree->board() != b)
+    {
+        m_UndoStack->push(new BoardUndoCommand(this,&f,m_openingTree->board(),m_openingTree->bEnd(),""));
+    }
+    doSetBoard(f,b,bEnd);
+}
+
+void OpeningTreeWidget::doSetBoard(Filter& f, const Board& b, bool bEnd)
 {
     m_openingBoardView->setBoard(b);
-    return m_openingTree->updateFilter(f, b, ui->filterGames->isChecked(), ui->sourceSelector->currentIndex()==1, bEnd);
+    m_openingTree->updateFilter(f, b, ui->filterGames->isChecked(), ui->sourceSelector->currentIndex()==1, bEnd);
 }
 
 void OpeningTreeWidget::saveConfig()
@@ -88,7 +118,8 @@ void OpeningTreeWidget::slotTreeUpdate()
     ui->progress->setValue(100);
     if (ui->filterGames->isChecked())
     {
-        emit signalTreeUpdated();
+        bool dbIsFilterSource = (ui->sourceSelector->currentIndex()<=1);
+        emit signalTreeUpdated(dbIsFilterSource);
     }
 }
 
@@ -99,17 +130,35 @@ void OpeningTreeWidget::slotTreeUpdateStarted()
 
 int OpeningTreeWidget::getFilterIndex(QString& name) const
 {
-    name = ui->sourceSelector->currentText();
-    return ui->sourceSelector->currentIndex();
+    int index = ui->sourceSelector->currentIndex();
+    if (index>1)
+        name = m_filePaths[ui->sourceSelector->currentIndex()-2];
+    else
+        name = ui->sourceSelector->currentText();
+    return index;
 }
 
 void OpeningTreeWidget::updateFilterIndex(QStringList files)
 {
+    m_filePaths.clear();
+    m_filePaths = files;
     disconnect(ui->sourceSelector, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSourceChanged()));
     QString current = ui->sourceSelector->currentText();
     ui->sourceSelector->clear();
+    QStringList baseNames;
+    foreach(QString filename, files)
+    {
+        QFileInfo fi(filename);
+        QString baseName = fi.baseName();
+        if (DatabaseInfo::IsBook(filename))
+        {
+            baseName += tr(" (Book)");
+        }
+        baseNames.append(baseName);
+    }
+
     QStringList allFiles;
-    allFiles << tr("Database") << tr("Filter") << files;
+    allFiles << tr("Database") << tr("Filter") << baseNames;
     ui->sourceSelector->insertItems(0, allFiles);
     if (allFiles.contains(current))
     {
@@ -119,7 +168,6 @@ void OpeningTreeWidget::updateFilterIndex(QStringList files)
     else
     {
         ui->sourceSelector->setCurrentIndex(0);
-        ui->filterGames->setEnabled(true);
     }
     connect(ui->sourceSelector, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSourceChanged()));
 }
@@ -131,12 +179,7 @@ bool OpeningTreeWidget::shouldAddMove() const
 
 void OpeningTreeWidget::slotSourceChanged()
 {
+    m_UndoStack->clear();
     m_openingTree->cancel();
-    bool enableFilter = (ui->sourceSelector->currentIndex()<=1);
-    if (!enableFilter)
-    {
-        ui->filterGames->setChecked(false);
-    }
-    ui->filterGames->setEnabled(enableFilter);
     emit signalSourceChanged();
 }
