@@ -12,6 +12,11 @@
 #include <QHeaderView>
 #include <QMenu>
 
+#if defined(_MSC_VER) && defined(_DEBUG)
+#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__ )
+#define new DEBUG_NEW
+#endif // _MSC_VER
+
 TableView::TableView(QWidget *parent)
     : QTableView(parent)
 {
